@@ -35,12 +35,15 @@ History:
 * catch variables that have only one string or similar,
   Jan 2021, Matthias Cuntz
 * added tooltip to dimensions, Jan 2021, Matthias Cuntz
+* added analyse_netcdf from ncvmain, Jan 2021, Matthias Cuntz
+* use dlblval instead of dlbl to set dimension labels, Jan 2021, Matthias Cuntz
 
 .. moduleauthor:: Matthias Cuntz
 
 The following methods are provided:
 
 .. autosummary::
+   analyse_netcdf
    get_miss
    get_slice_miss
    set_dim_lat
@@ -55,7 +58,7 @@ from __future__ import absolute_import, division, print_function
 import tkinter as tk
 import numpy as np
 from .ncvutils import DIMMETHODS, get_slice, set_miss, spinbox_values
-from .ncvutils import vardim2var
+from .ncvutils import SEPCHAR, vardim2var, zip_dim_name_length
 import netCDF4 as nc
 # nc.default_fillvals but with keys as variables['var'].dtype
 nctypes = [ np.dtype(i) for i in nc.default_fillvals ]
@@ -63,11 +66,375 @@ ncfill  = dict(zip(nctypes, list(nc.default_fillvals.values())))
 ncfill.update({np.dtype('O'): np.nan})
 
 
-__all__ = ['get_miss', 'get_slice_miss',
+__all__ = ['analyse_netcdf', 'get_miss', 'get_slice_miss',
            'set_dim_lat', 'set_dim_lon', 'set_dim_var',
            'set_dim_x', 'set_dim_y', 'set_dim_y2', 'set_dim_z']
 
 
+#
+# Analyse netcdf file
+#
+
+def analyse_netcdf(self):
+    """
+    Analyse a netcdf file for the unlimited dimension, calculating datetime,
+    variables, latitudes/longitudes variables and dimensions.
+
+    Parameters
+    ----------
+    self : class
+        ncvue class
+
+    Returns
+    -------
+    Set variables:
+        self.dunlim,
+        self.time, self.tname, self.tvar, self.dtime,
+        self.cols,
+        self.latvar, self.lonvar, self.latdim, self.londim
+
+    Examples
+    --------
+    >>> analyse_netcdf(self)
+    """
+    import datetime as dt
+    try:
+        import cftime as cf
+    except ModuleNotFoundError:
+        import netCDF4 as cf
+    #
+    # search unlimited dimension
+    self.dunlim = ''
+    for dd in self.fi.dimensions:
+        if self.fi.dimensions[dd].isunlimited():
+            self.dunlim = dd
+            break
+    #
+    # search for time variable and make datetime variable
+    self.time  = None
+    self.tname = ''
+    self.tvar  = ''
+    self.dtime = None
+    for vv in self.fi.variables:
+        isunlim = False
+        if self.dunlim:
+            if vv.lower() == self.fi.dimensions[self.dunlim].name.lower():
+                isunlim = True
+        if ( isunlim or vv.lower().startswith('time_') or
+             (vv.lower() == 'time') or (vv.lower() == 'datetime') or
+             (vv.lower() == 'date') ):
+            self.tvar = vv
+            if vv.lower() == 'datetime':
+                self.tname = 'date'
+            else:
+                self.tname = 'datetime'
+            try:
+                tunit = self.fi.variables[self.tvar].units
+            except AttributeError:
+                tunit = ''
+            # assure 01, etc. if values < 10
+            if tunit.find('since') > 0:
+                tt = tunit.split()
+                dd = tt[2].split('-')
+                dd[0] = ('000'+dd[0])[-4:]
+                dd[1] = ('0'+dd[1])[-2:]
+                dd[2] = ('0'+dd[1])[-2:]
+                tt[2] = '-'.join(dd)
+                tunit = ' '.join(tt)
+            try:
+                tcal = self.fi.variables[self.tvar].calendar
+            except AttributeError:
+                tcal = 'standard'
+            time = self.fi.variables[self.tvar][:]
+            # time dimension "day as %Y%m%d.%f" from cdo.
+            if ' as ' in tunit:
+                itunit = tunit.split()[2]
+                dtime = []
+                for tt in time:
+                    stt = str(tt).split('.')
+                    sstt = ('00'+stt[0])[-8:] + '.' + stt[1]
+                    dtime.append(dt.datetime.strptime(sstt, itunit))
+                ntime = cf.date2num(dtime,
+                                    'days since 0001-01-01 00:00:00')
+                self.dtime = cf.num2date(ntime,
+                                         'days since 0001-01-01 00:00:00')
+            else:
+                try:
+                    self.dtime = cf.num2date(time, tunit,
+                                             calendar=tcal)
+                except ValueError:
+                    self.dtime = None
+            if self.dtime is not None:
+                ntime = len(self.dtime)
+                if (tcal == '360_day'):
+                    ndays = [360.]*ntime
+                elif (tcal == '365_day'):
+                    ndays = [365.]*ntime
+                elif (tcal == 'noleap'):
+                    ndays = [365.]*ntime
+                elif (tcal == '366_day'):
+                    ndays = [366.]*ntime
+                elif (tcal == 'all_leap'):
+                    ndays = [366.]*ntime
+                else:
+                    ndays = [ 365. +
+                              float((((t.year%4) == 0) &
+                                     ((t.year%100) != 0)) |
+                                    ((t.year%400) == 0))
+                              for t in self.dtime ]
+                self.dtime = np.array([
+                    t.year +
+                    (t.dayofyr-1 + t.hour / 24. +
+                     t.minute / 1440 + t.second / 86400.) / ndays[i]
+                    for i, t in enumerate(self.dtime) ])
+            # make datetime variable
+            if self.time is None:
+                try:
+                    self.time = cf.num2date(
+                        time, tunit, calendar=tcal,
+                        only_use_cftime_datetimes=False,
+                        only_use_python_datetimes=True)
+                except:
+                    self.time = None
+            if self.time is None:
+                try:
+                    self.time = cf.num2date(time, tunit,
+                                            calendar=tcal)
+                except:
+                    self.time = None
+            if self.time is None:
+                # if not possible use decimal year
+                self.time = self.dtime
+            if self.time is None:
+                # could not interpret time at all,
+                # e.g. if units = "months since ..."
+                self.time  = time
+                self.dtime = time
+            break
+    #
+    # construct list of variable names with dimensions
+    if self.time is not None:
+        addt = [
+            self.tname + ' ' + SEPCHAR +
+            str(tuple(zip_dim_name_length(self.fi.variables[self.tvar])))]
+        self.cols += addt
+    ivars = []
+    for vv in self.fi.variables:
+        # ss = self.fi.variables[vv].shape
+        ss = tuple(zip_dim_name_length(self.fi.variables[vv]))
+        self.maxdim = max(self.maxdim, len(ss))
+        ivars.append((vv, ss, len(ss)))
+    self.cols += sorted([ vv[0] + ' ' + SEPCHAR + str(vv[1])
+                          for vv in ivars ])
+    #
+    # search for lat/lon variables
+    self.latvar = ''
+    self.lonvar = ''
+    # first sweep: *name must be "latitude" and
+    #              units must be "degrees_north"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            try:
+                sname = self.fi.variables[vv].standard_name
+            except AttributeError:
+                try:
+                    sname = self.fi.variables[vv].long_name
+                except AttributeError:
+                    sname = self.fi.variables[vv].name
+            if sname.lower() == 'latitude':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_north':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            try:
+                sname = self.fi.variables[vv].standard_name
+            except AttributeError:
+                try:
+                    sname = self.fi.variables[vv].long_name
+                except AttributeError:
+                    sname = self.fi.variables[vv].name
+            if sname.lower() == 'longitude':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_east':
+                    self.lonvar = vv
+    # second sweep: name must start with lat and
+    #               units must be "degrees_north"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            if sname[0:3].lower() == 'lat':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_north':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            if sname[0:3].lower() == 'lon':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_east':
+                    self.lonvar = vv
+    # third sweep: name must contain lat and
+    #              units must be "degrees_north"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            sname = sname.lower()
+            if sname.find('lat') >= 0:
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_north':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            sname = sname.lower()
+            if sname.find('lon') >= 0:
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees_east':
+                    self.lonvar = vv
+    # fourth sweep: same as first but units can be "degrees"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            try:
+                sname = self.fi.variables[vv].standard_name
+            except AttributeError:
+                try:
+                    sname = self.fi.variables[vv].long_name
+                except AttributeError:
+                    sname = self.fi.variables[vv].name
+            if sname.lower() == 'latitude':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            try:
+                sname = self.fi.variables[vv].standard_name
+            except AttributeError:
+                try:
+                    sname = self.fi.variables[vv].long_name
+                except AttributeError:
+                    sname = self.fi.variables[vv].name
+            if sname.lower() == 'longitude':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.lonvar = vv
+    # fifth sweep: same as second but units can be "degrees"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            if sname[0:3].lower() == 'lat':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            if sname[0:3].lower() == 'lon':
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.lonvar = vv
+    # sixth sweep: same as third but units can be "degrees"
+    if not self.latvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            sname = sname.lower()
+            if sname.find('lat') >= 0:
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.latvar = vv
+    if not self.lonvar:
+        for vv in self.fi.variables:
+            sname = self.fi.variables[vv].name
+            sname = sname.lower()
+            if sname.find('lon') >= 0:
+                try:
+                    sunit = self.fi.variables[vv].units
+                except AttributeError:
+                    sunit = ''
+                if sunit.lower() == 'degrees':
+                    self.lonvar = vv
+    #
+    # determine lat/lon dimensions
+    self.latdim = ''
+    self.londim = ''
+    if self.latvar:
+        latshape = self.fi.variables[self.lonvar].shape
+        if (len(latshape) < 1) or (len(latshape) > 2):
+            estr  = 'Something went wrong determining lat/lon:'
+            estr += ' latitude variable is not 1D or 2D.'
+            print(estr)
+            estr = 'latitude variable with dimensions:'
+            ldim = self.fi.variables[self.latvar].dimensions
+            print(estr, self.latvar, ldim)
+            estr = 'longitude variable with dimensions:'
+            ldim = self.fi.variables[self.lonvar].dimensions
+            print(estr, self.lonvar, ldim)
+            self.latvar = ''
+            self.lonvar = ''
+        else:
+            self.latdim = self.fi.variables[self.latvar].dimensions[0]
+    if self.lonvar:
+        lonshape = self.fi.variables[self.lonvar].shape
+        if len(lonshape) == 1:
+            self.londim = self.fi.variables[self.lonvar].dimensions[0]
+        elif len(lonshape) == 2:
+            self.londim = self.fi.variables[self.lonvar].dimensions[1]
+        else:
+            estr  = 'Something went wrong determining lat/lon:'
+            estr += ' longitude variable is not 1D or 2D.'
+            print(estr)
+            estr = 'latitude variable with dimensions:'
+            ldim = self.fi.variables[self.latvar].dimensions
+            print(estr, self.latvar, ldim)
+            estr = 'longitude variable with dimensions:'
+            ldim = self.fi.variables[self.lonvar].dimensions
+            print(estr, self.lonvar, ldim)
+            self.latvar = ''
+            self.lonvar = ''
+    #
+    # add units to lat/lon name
+    if self.latvar:
+        idim = tuple(zip_dim_name_length(self.fi.variables[self.latvar]))
+        self.latvar = self.latvar + ' ' + SEPCHAR + str(idim)
+    if self.lonvar:
+        idim = tuple(zip_dim_name_length(self.fi.variables[self.lonvar]))
+        self.lonvar = self.lonvar + ' ' + SEPCHAR + str(idim)
+
+
 #
 # Get list of missing values
 #
@@ -184,7 +551,7 @@ def set_dim_lat(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.latd[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.latdlbl[i].set(str(i))
+        self.latdlblval[i].set(str(i))
         self.latdtip[i].set("")
     lat = self.lat.get()
     if lat != '':
@@ -201,7 +568,7 @@ def set_dim_lat(self):
                 self.latdval[i].set('all')
             else:
                 self.latdval[i].set(0)
-            self.latdlbl[i].set(ll.dimensions[i])
+            self.latdlblval[i].set(ll.dimensions[i])
             if ll.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     ll.shape[i]-1)
@@ -235,7 +602,7 @@ def set_dim_lon(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.lond[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.londlbl[i].set(str(i))
+        self.londlblval[i].set(str(i))
         self.londtip[i].set("")
     lon = self.lon.get()
     if lon != '':
@@ -252,7 +619,7 @@ def set_dim_lon(self):
                 self.londval[i].set('all')
             else:
                 self.londval[i].set(0)
-            self.londlbl[i].set(ll.dimensions[i])
+            self.londlblval[i].set(ll.dimensions[i])
             if ll.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     ll.shape[i]-1)
@@ -290,7 +657,7 @@ def set_dim_var(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.vd[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.vdlbl[i].set(str(i))
+        self.vdlblval[i].set(str(i))
         self.vdtip[i].set("")
     v = self.v.get()
     if v != '':
@@ -308,7 +675,7 @@ def set_dim_var(self):
                                   state=tk.NORMAL)
                 nall += 1
                 self.vdval[i].set('all')
-                self.vdlbl[i].set(vv.dimensions[i])
+                self.vdlblval[i].set(vv.dimensions[i])
                 if vv.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         vv.shape[i]-1)
@@ -325,7 +692,7 @@ def set_dim_var(self):
                                   state=tk.NORMAL)
                 nall += 1
                 self.vdval[i].set('all')
-                self.vdlbl[i].set(vv.dimensions[i])
+                self.vdlblval[i].set(vv.dimensions[i])
                 if vv.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         vv.shape[i]-1)
@@ -343,7 +710,7 @@ def set_dim_var(self):
                 (nall <= 1) and (vv.shape[i] > 1)):
                 nall += 1
                 self.vdval[i].set('all')
-                self.vdlbl[i].set(vv.dimensions[i])
+                self.vdlblval[i].set(vv.dimensions[i])
                 if vv.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         vv.shape[i]-1)
@@ -355,7 +722,7 @@ def set_dim_var(self):
             elif ((vv.dimensions[i] != self.latdim) and
                   (vv.dimensions[i] != self.londim)):
                 self.vdval[i].set(0)
-                self.vdlbl[i].set(vv.dimensions[i])
+                self.vdlblval[i].set(vv.dimensions[i])
                 if vv.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         vv.shape[i]-1)
@@ -393,7 +760,7 @@ def set_dim_x(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.xd[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.xdlbl[i].set(str(i))
+        self.xdlblval[i].set(str(i))
         self.xdtip[i].set("")
     x = self.x.get()
     if x != '':
@@ -410,7 +777,7 @@ def set_dim_x(self):
                               state=tk.NORMAL)
             nall += 1
             self.xdval[i].set('all')
-            self.xdlbl[i].set(xx.dimensions[i])
+            self.xdlblval[i].set(xx.dimensions[i])
             if xx.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     xx.shape[i]-1)
@@ -429,7 +796,7 @@ def set_dim_x(self):
                     self.xdval[i].set('all')
                 else:
                     self.xdval[i].set(0)
-                self.xdlbl[i].set(xx.dimensions[i])
+                self.xdlblval[i].set(xx.dimensions[i])
                 if xx.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         xx.shape[i]-1)
@@ -467,7 +834,7 @@ def set_dim_y(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.yd[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.ydlbl[i].set(str(i))
+        self.ydlblval[i].set(str(i))
         self.ydtip[i].set("")
     y = self.y.get()
     if y != '':
@@ -484,7 +851,7 @@ def set_dim_y(self):
                               state=tk.NORMAL)
             nall += 1
             self.ydval[i].set('all')
-            self.ydlbl[i].set(yy.dimensions[i])
+            self.ydlblval[i].set(yy.dimensions[i])
             if yy.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     yy.shape[i]-1)
@@ -503,7 +870,7 @@ def set_dim_y(self):
                     self.ydval[i].set('all')
                 else:
                     self.ydval[i].set(0)
-                self.ydlbl[i].set(yy.dimensions[i])
+                self.ydlblval[i].set(yy.dimensions[i])
                 if yy.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         yy.shape[i]-1)
@@ -541,7 +908,7 @@ def set_dim_y2(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.y2d[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.y2dlbl[i].set(str(i))
+        self.y2dlblval[i].set(str(i))
         self.y2dtip[i].set("")
     y2 = self.y2.get()
     if y2 != '':
@@ -558,7 +925,7 @@ def set_dim_y2(self):
                                state=tk.NORMAL)
             nall += 1
             self.y2dval[i].set('all')
-            self.y2dlbl[i].set(yy2.dimensions[i])
+            self.y2dlblval[i].set(yy2.dimensions[i])
             if yy2.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     yy2.shape[i]-1)
@@ -577,7 +944,7 @@ def set_dim_y2(self):
                     self.y2dval[i].set('all')
                 else:
                     self.y2dval[i].set(0)
-                self.y2dlbl[i].set(yy2.dimensions[i])
+                self.y2dlblval[i].set(yy2.dimensions[i])
                 if yy2.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         yy2.shape[i]-1)
@@ -616,7 +983,7 @@ def set_dim_z(self):
     # reset dimensions
     for i in range(self.maxdim):
         self.zd[i].config(values=(0,), width=1, state=tk.DISABLED)
-        self.zdlbl[i].set(str(i))
+        self.zdlblval[i].set(str(i))
         self.zdtip[i].set("")
     z = self.z.get()
     if z != '':
@@ -633,7 +1000,7 @@ def set_dim_z(self):
                               state=tk.NORMAL)
             nall += 1
             self.zdval[i].set('all')
-            self.zdlbl[i].set(zz.dimensions[i])
+            self.zdlblval[i].set(zz.dimensions[i])
             if zz.shape[i] > 1:
                 tstr  = "Specific dimension value: 0-{:d}\n".format(
                     zz.shape[i]-1)
@@ -652,7 +1019,7 @@ def set_dim_z(self):
                     self.zdval[i].set('all')
                 else:
                     self.zdval[i].set(0)
-                self.zdlbl[i].set(zz.dimensions[i])
+                self.zdlblval[i].set(zz.dimensions[i])
                 if zz.shape[i] > 1:
                     tstr  = "Specific dimension value: 0-{:d}\n".format(
                         zz.shape[i]-1)
